﻿using Noesis;
using Canvas = System.Windows.Controls.Canvas;

namespace Assets.BrightHue.Control
{
    public class ArcControl : Canvas
    {
        public static DependencyProperty DirectionProperty = DependencyProperty.Register(
                "Direction",
                typeof(int),
                typeof(ArcControl),
                new FrameworkPropertyMetadata(0, FrameworkOptions.AffectsRender, PropertyChangedCallback)
            );

        public static DependencyProperty ArcWidthProperty = DependencyProperty.Register(
                "ArcWidth",
                typeof(int),
                typeof(ArcControl),
                new FrameworkPropertyMetadata(0, FrameworkOptions.AffectsRender, PropertyChangedCallback)
            );

        public static DependencyProperty StrokeProperty = DependencyProperty.Register(
                "Stroke",
                typeof(Color),
                typeof(ArcControl),
                new FrameworkPropertyMetadata(Colors.Transparent, FrameworkOptions.AffectsRender, PropertyChangedCallback)
            );

        public static DependencyProperty FillProperty = DependencyProperty.Register(
                "Fill",
                typeof(Color),
                typeof(ArcControl),
                new FrameworkPropertyMetadata(Colors.Black, FrameworkOptions.AffectsRender, PropertyChangedCallback)
            );


        private static void PropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs dependencyPropertyChangedEventArgs)
        {
            Refresh(dependencyObject);
        }

        private static void Refresh(DependencyObject dependencyObject)
        {
            var arcControl = dependencyObject as ArcControl;
            if (arcControl == null)
            {
                return;
            }
            var path = new Path();
            path.Stroke = new SolidColorBrush(arcControl.Stroke);
            path.Fill = new SolidColorBrush(arcControl.Fill);

            arcControl.Children.Clear();


            arcControl.Children.Add(path);
            var arcLength = (float)arcControl.Width / 2f;
            var startAngle = arcControl.Direction;
            var arcWidth = arcControl.ArcWidth;

            var pieSlice = ShapeTools.CreatePieSlice(arcLength, startAngle, arcWidth);
            path.Data = pieSlice;
        }

        public int Direction
        {
            get { return (int)GetValue(DirectionProperty); }
            set { SetValue(DirectionProperty, value); }
        }

        public int ArcWidth
        {
            get { return (int)GetValue(ArcWidthProperty); }
            set { SetValue(ArcWidthProperty, value); }
        }

        public Color Stroke
        {
            get { return (Color)GetValue(StrokeProperty); }
            set { SetValue(StrokeProperty, value); }
        }

        public Color Fill
        {
            get { return (Color)GetValue(FillProperty); }
            set { SetValue(FillProperty, value); }
        }
    }
}
