﻿using System.Collections;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;
using Assets.BrightHue.Framework.Tools;
using Noesis;

namespace Assets.BrightHue.Control
{
    public class PieSlice : DependencyObject
    {
        public static DependencyProperty ColorProperty =
            DependencyProperty.Register("Color",
            typeof(Color),
            typeof(PieSlice),
            new FrameworkPropertyMetadata(Colors.Blue, FrameworkOptions.AffectsRender, PropertyChangedCallback));

        public static DependencyProperty PercentProperty =
            DependencyProperty.Register("Percent",
            typeof(int),
            typeof(PieSlice),
            new FrameworkPropertyMetadata(0, FrameworkOptions.AffectsRender, PropertyChangedCallback));

        private static void PropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs dependencyPropertyChangedEventArgs)
        {
            var pieSlice = dependencyObject as PieSlice;
            if (pieSlice != null)
            {
                if (pieSlice.Parent != null)
                {
                    pieSlice.Parent.Refresh();
                }
            }
        }

        public Color Color
        {
            get { return (Color)GetValue(ColorProperty); }
            set { SetValue(ColorProperty, value); }
        }

        public int Percent
        {
            get { return (int)GetValue(PercentProperty); }
            set { SetValue(PercentProperty, value); }
        }

        public PieChartControl Parent { get; set; }
    }

    public class PieChartControl : Canvas
    {
        //public static DependencyProperty PieSlicesProperty =
        //    DependencyProperty.Register("PieSlices",
        //    typeof(ObservableCollection<PieSlice>),
        //    typeof(PieChartControl),
        //    new FrameworkPropertyMetadata(new ObservableCollection<PieSlice>()));
        public ObservableCollection<PieSlice> _pieSlices = new ObservableCollection<PieSlice>();

        public ObservableCollection<PieSlice> PieSlices
        {
            get
            {
                return _pieSlices;
            }
            set
            {
                _pieSlices = value;                
            }
        }
        //public ObservableCollection<PieSlice> PieSlices
        //{
        //    get { return GetValue(PieSlicesProperty) as ObservableCollection<PieSlice>; }
        //    set { SetValue(PieSlicesProperty, value); }
        //}


        public void OnPostInit()
        {
            if (PieSlices != null)
            {
                PieSlices.CollectionChanged += PieSlicesOnCollectionChanged;
            }
            Refresh();
        }

        private void PieSlicesOnCollectionChanged(object sender, NotifyCollectionChangedEventArgs notifyCollectionChangedEventArgs)
        {
            if (notifyCollectionChangedEventArgs.NewItems != null)
            {
                var items = notifyCollectionChangedEventArgs.NewItems;
                foreach (var item in items)
                {
                    var pieSlice = item as PieSlice;
                    if (pieSlice != null)
                    {
                        pieSlice.Parent = this;
                    }
                }
            }
            Refresh();
        }

        public void Refresh()
        {
            var sum = PieSlices.Sum(o => o.Percent);
            int angle = 0;
            Children.Clear();
            for (int index = 0; index < PieSlices.Count; index++)
            {
                var pieSlice = PieSlices[index];
                var path = new Path();
                var width = (int)(pieSlice.Percent * 360f / sum);
                if (index + 1 == PieSlices.Count)
                {
                    if (angle + width != 360)
                    {
                        width = 360 - angle;
                    }
                }
                path.Fill = new SolidColorBrush(pieSlice.Color);
                path.Data = ShapeTools.CreatePieSlice((float)Width, angle, width);
                Children.Add(path);
                angle += width;
            }
        }
    }
}
