﻿using System;
using Assets.BrightHue.Framework.Tools;
using Noesis;
using UnityEngine;

namespace Assets.BrightHue.Control
{
    public class ShapeTools
    {
        public static float ToRadians(float angleInDegrees)
        {
            double radian = Math.PI * angleInDegrees / 180;
            return Convert.ToSingle(radian);
        }
        public static Vector2 GetPointTheta(float distance, Vector2 origin, float angleInDegrees)
        {
            double radian = ToRadians(angleInDegrees);
            double x = origin.x + distance * Math.Cos(radian);
            double y = origin.y + distance * Math.Sin(radian);
            return new Vector2((float)x, (float)y);
        }
        public static StreamGeometry CreatePieSlice(float arcLength, int startAngle, int arcWidth)
        {
            var origin = new Vector2(arcLength, arcLength);
            var theta = GetPointTheta(arcLength, origin, startAngle);
            var originPoint = new Point(arcLength, arcLength);
            var geometry = new StreamGeometry();
            using (var ctx = geometry.Open())
            {
                ctx.BeginFigure(originPoint, true);
                var lastPoint = new Point(theta.x, theta.y);
                ctx.LineTo(lastPoint);
                for (int x = 1; x <= arcWidth; x++)
                {
                    var newTheta = MathTools.GetPointTheta(arcLength, origin, startAngle + x);
                    var newPoint = new Point(newTheta.x, newTheta.y);
                    ctx.LineTo(newPoint);
                }
                ctx.LineTo(originPoint);
            }
            return geometry;
        }
    }
}
